﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;

namespace WpfTest.Model
{
    public static class Analyses
    {
        static double startX = 1;
        static double Interval = 1;
        /// <summary>
        /// 对于给定的原始点系列进行 点补间 ，返回补间点集合
        /// </summary>
        /// <param name="pf">原始点集合</param>
        /// <param name="interval">补间间隔</param>
        /// <param name="scaleZ">Z值扩展倍率</param>
        /// <returns></returns>
        public static ScaleProfile Interpolation(Profile pf, double interval, double scaleZ)
        {
            //原始点集合为空，返回null
            if(pf.Values.Count<=0)
            {
                return null;
            }
            //
            ScaleProfile scaleProfile = new ScaleProfile();
            Method method = new Method();
            List<ScaleProfilePoint> listScaleProfilePoint = new List<ScaleProfilePoint>();
            double distance = 0;
            bool flag = true;
            Point pTmp = new Point();
            //z扩展scaleZ倍率
            for (int i = 0; i < pf.Values.Count; i++)
            {
                pf.Values[i].Value *= scaleZ;
            }

            //原始点起点是第一个补间点，
            ScaleProfilePoint scaleProfilePoint = new ScaleProfilePoint();
            scaleProfilePoint.ValueX = startX;
            scaleProfilePoint.ValueZ = pf.Values[0].Value;
            listScaleProfilePoint.Add(scaleProfilePoint);
            //从原始点起点开始寻找补间点，到原始点终点结束循环，
            //原始点终点与相邻补间点距离小于补间距离则舍弃并循环结束.
            for (int i = 0; i < pf.Values.Count; i++)
            {
                ScaleProfilePoint scaleProfilePointTmp = new ScaleProfilePoint();
                
                //list集合记录的最后一个补间点
                Point p1 = new Point(listScaleProfilePoint[listScaleProfilePoint.Count - 1].ValueX, listScaleProfilePoint[listScaleProfilePoint.Count - 1].ValueZ);
                if(flag)
                {
                    pTmp = p1;
                }
                //当前比较对象（原始点）
                Point p2 = new Point(startX + i * Interval, pf.Values[i].Value);
                distance += method.distanceTwoPoints(pTmp, p2);
                //list最后补间点与当前原始点距离小于interval
                if (distance < interval)
                {
                    pTmp = p2;
                    flag = false;
                }
                //list最后补间点与原始点距离小于interval时，找到了补间点区间，求出补间点
                else if(flag == false)
                {
                    scaleProfilePointTmp.ValueX = method.distanceByTwopoints(pTmp, p2, method.distanceTwoPoints(pTmp, p2) - distance + interval).X;
                    scaleProfilePointTmp.ValueZ = method.distanceByTwopoints(pTmp, p2, method.distanceTwoPoints(pTmp, p2) - distance + interval).Y;
                    listScaleProfilePoint.Add(scaleProfilePointTmp);
                    flag = true;
                    distance = 0;
                    i--;
                }
                //list最后补间点与当前原始点距离大于interval
                else
                {
                    scaleProfilePointTmp.ValueX = method.distanceByTwopoints(p1, p2, interval).X;
                    scaleProfilePointTmp.ValueZ = method.distanceByTwopoints(p1, p2, interval).Y;
                    listScaleProfilePoint.Add(scaleProfilePointTmp);
                    distance = 0;
                    i--;
                }
            }
            //完善返回值信息
            scaleProfile.Values = listScaleProfilePoint;
            scaleProfile.ScaleZ = scaleZ;
            
            return scaleProfile;
        }
        /// <summary>
        /// 最小二乘法获得近似线集合，二乘法计算单位（在给定点数中应用最小二乘法，点的集合成为二乘法最小单位）
        /// </summary>
        /// <param name="pf">补间点集合</param>
        /// <param name="pointCnt">二乘法最小计算单位的点数</param>
        /// <param name="overwrapCnt">二乘法计算单位间的重复点数</param>
        /// <returns></returns>
        public  static List<ApproximateLine> FindApproximateLines (ScaleProfile pf, int pointCnt,int overwrapCnt )
        {
            Method method = new Method();
            List<ApproximateLine> listAppr = new List<ApproximateLine>();
            //i是根据补间点数量、ponitCnt、overwrapCnt可以拟合的近似线数量
            for (int i = 1; i < pf.Values.Count / (pointCnt - overwrapCnt);i++ )
            {
                List<Point> listPoints = new List<Point>();
                //提取出拟合近似线的补间点集合
                for (int j = 0; j < pointCnt;j++ )
                {
                    Point point = new Point(pf.Values[(i - 1) * (pointCnt - overwrapCnt) + j].ValueX, pf.Values[(i - 1) * (pointCnt - overwrapCnt) + j].ValueZ);
                    listPoints.Add(point);
                }
                //拟合近似线
                ApproximateLine appr = method.calApproximateLine(listPoints);
                listAppr.Add(appr);
            }
            return listAppr;
        }
        /// <summary>
        /// 重计算拟合线
        /// </summary>
        /// <param name="lines">重计算之前的拟合线集合</param>
        /// <param name="lineThreshold">判断是否重计算的阈值</param>
        /// <param name="horizontalThreshold"></param>
        /// <returns></returns>
        public static List<ApproximateLine> ConnectLines(List<ApproximateLine> lines, double lineThreshold, double horizontalThreshold)
        {
            List<ApproximateLine> listaapr = new List<ApproximateLine>();
            Method method = new Method();
            //flag==true标志for从continue退出
            bool flag = false;
            for (int i = 0; i < lines.Count;i++ )
            {
                method.ListTmp.Add(lines[i]);
                //小于阈值继续加入集合判断，大于阈值计算新近似线
                if( method.listApprThreshold() < lineThreshold )
                {
                    flag = true;
                    continue;
                }
                else
                {
                    //计算新近似线
                    ApproximateLine apprline = method.listLinesToLine();
                    listaapr.Add(apprline);
                    //清空集合
                    method.ListTmp.Clear();
                    flag = false;
                }
            }
            //收集最后未处理的近似线
            if (flag)
            {
                ApproximateLine apprline = method.listLinesToLine();
                listaapr.Add(apprline);
                //清空集合
                method.ListTmp.Clear();
            }
            return listaapr;
        }
    }
}
