﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;

namespace WpfTest.Model
{
    public class Method
    {
        /// <summary>
        /// 生成随机原始点
        /// </summary>
        /// <returns></returns>
        public Profile newProfile()
        { 
            Profile profile =new Profile();
            Random rand = new Random();
            List<ProfilePoint> listpfopoint = new List<ProfilePoint>();
            for (int y = 0; y < 700; y++)
            {
                ProfilePoint profilePoint = new ProfilePoint();
                profilePoint.Value = rand.Next(1,100);
                profilePoint.Type = 1;
                listpfopoint.Add(profilePoint);
            }
            profile.Interval = 1;
            profile.Values = listpfopoint;
            return profile;
        }
        /// <summary>
        /// 正弦波原始点
        /// </summary>
        /// <returns></returns>
        public Profile newSinProfile()
        {
            Profile profile = new Profile();
            List<ProfilePoint> list = new List<ProfilePoint>();
            for(int i =0;i<600;i++)
            {
                ProfilePoint profilePoint = new ProfilePoint();
                profilePoint.Value =76 * (1 - Math.Sin(i * Math.PI / 92));
                profilePoint.Type = 1;
                list.Add(profilePoint);
            }
            profile.Interval = 1;
            profile.Values = list;
            return profile;
        }
        /// <summary>
        /// 两点之间距离
        /// </summary>
        /// <param name="p1">点1</param>
        /// <param name="p2">点2</param>
        /// <returns></returns>
        public double distanceTwoPoints(Point p1,Point p2)
        {
            double dis = 0;
            dis = Math.Sqrt(Math.Pow(p1.X - p2.X, 2) + Math.Pow(p1.Y - p2.Y, 2));
            return dis;
        }
        /// <summary>
        /// 找到起点到终点之间的某个点p，p与起点的距离为distance
        /// </summary>
        /// <param name="p1">起点</param>
        /// <param name="p2">终点</param>
        /// <param name="distance">p与起点的距离为distance</param>
        /// <returns></returns>
        public Point distanceByTwopoints(Point p1,Point p2,double distance)
        {
            Point p = new Point();

            //y = k*x + b
            double k = (p1.Y - p2.Y) / (p1.X - p2.X);
            double b = p2.Y - k * p2.X;

            //三角形相似变换
            //distance/distance<p1,p2> = (x - p1.x)/(p2.x - 1.x)
            p.X = distance * (p2.X - p1.X) / distanceTwoPoints(p1, p2) + p1.X;
            p.Y = p.X * k + b;

            return p;
        }
        /// <summary>
        /// 最小二乘法计算回归直线
        /// </summary>
        /// <param name="listPoints">最小二乘法计算单位</param>
        /// <returns></returns>
        public ApproximateLine calApproximateLine(List<Point> listPoints)
        {
            ApproximateLine appr = new ApproximateLine();
            double total = 0;       //临时变量
            double averageX = 0;    //X平均值
            double averageY = 0;    //Y平均值
            double dispersion = 0;  //协方差
            double b = 0;  //斜率
            double a = 0;  //y轴截距
            //x平均值计算
            total = 0;
            for (int i = 0; i < listPoints.Count; i++)
            {
                total += listPoints[i].X;
            }
            averageX = total / listPoints.Count;
            //y平均值计算
            total = 0;
            for (int i = 0; i < listPoints.Count; i++)
            {
                total += listPoints[i].Y;
            }
            averageY = total / listPoints.Count;
            //协方差计算
            total = 0;
            for (int i = 0; i < listPoints.Count; i++)
            {
                total += ((listPoints[i].Y - averageY)*(listPoints[i].X - averageX));
            }
            dispersion = total / listPoints.Count;
            //斜率计算
            total = 0;
            double tmp = 0;
            for (int i = 0; i < listPoints.Count; i++)
            {
                total += listPoints[i].Y * listPoints[i].X;
                tmp += Math.Pow(listPoints[i].X, 2);
            }
            b = (total - listPoints.Count*averageX*averageY)/(tmp - listPoints.Count*averageX*averageX);
            //截距计算
            a = averageY - b * averageX;
            //确定起止点坐标
            ScaleProfilePoint p1 = new ScaleProfilePoint();
            ScaleProfilePoint p2 = new ScaleProfilePoint();
            p1.Type = 1;
            p1.ValueX = listPoints[0].X;
            p1.ValueZ = a + b * p1.ValueX;
            p2.Type = 1;
            p2.ValueX = listPoints[listPoints.Count-1].X;
            p2.ValueZ = a + b * p2.ValueX;
            //填充返回值
            appr.Dispersion = dispersion;
            appr.Horizontal = true;
            appr.StartPnt = p1;
            appr.EndPnt = p2;
            return appr;
        }

        #region
        //用于拟合直线的集合阈值判断
        List<ApproximateLine> listTmp = new List<ApproximateLine>();

        public List<ApproximateLine> ListTmp
        {
            get { return listTmp; }
            set { listTmp = value; }
        }
        /// <summary>
        /// 计算lineThreshold
        /// </summary>
        /// <returns></returns>
        public double listApprThreshold()
        {
            //记录集合最大、最小斜率差
            double max = 0;
            double min = 0;
            //计算集合最大、最小斜率差
            for (int i = 0; i < listTmp.Count;i++ )
            {
                double tmp = (listTmp[i].EndPnt.ValueZ - listTmp[i].StartPnt.ValueZ) / (listTmp[i].EndPnt.ValueX - listTmp[i].StartPnt.ValueX);
                if (0 == i)
                {
                    max = tmp;
                    min = tmp;
                }
                else
                {
                    max = tmp > max ? tmp : max;
                    min = tmp < min ? tmp : min;
                }
            }
            //返回集合最大最小斜率之差
            return (max - min);
        }
        /// <summary>
        /// 拟合新近似线
        /// </summary>
        /// <returns></returns>
        public ApproximateLine listLinesToLine()
        {
            ApproximateLine apprLine = new ApproximateLine();
            List<Point> listPoints = new List<Point>();
            //计算点集合
            for (int i = 0; i < listTmp.Count; i++)
            {
                Point p1 = new Point(listTmp[i].StartPnt.ValueX, listTmp[i].StartPnt.ValueZ);
                Point p2 = new Point(listTmp[i].EndPnt.ValueX, listTmp[i].EndPnt.ValueZ);
                listPoints.Add(p1);
                listPoints.Add(p2);
            }
            //计算近似线
            apprLine = calApproximateLine(listPoints);
            //返回新近似线
            return apprLine;
        }
        #endregion

    }
}
